import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/ui/component/nui_row.dart';
import 'package:nui_form/nuif_form.dart';
import 'package:nui_form/nuif_form_component.dart';
import 'package:nui_core/code/nui_code_util.dart';

class NUIFRowItem extends NUIFormContainer{
  NUIFRowItem({
    @required List<NUIFormComponent> items,
    @required String inpId,
    this.divider:0.0,
    this.padding,
    double width,
    double height,
    Alignment align,
    this.crossAxisAlignment: CrossAxisAlignment.start,
    EdgeInsets margin,
    double weight}): super(items: items, inpId : inpId, inpType: ComponentType.ROW, width: width, height: height, weight: weight, margin: margin, align: align);
  double divider;
  EdgeInsets padding;
  CrossAxisAlignment crossAxisAlignment;

  @override
  Widget toWidget({AllButtonClickListener allButtonClickListener, AllTextChangeListener allTextChangeListener, AllItemSelectListener allItemSelectListener, Key key}) {
    return NUIFRowLayout(item: this, key: key, allTextChangeListener: allTextChangeListener, allButtonClickListener: allButtonClickListener, allItemSelectListener: allItemSelectListener,);
  }
}

class NUIFRowLayout extends StatefulWidget {
  NUIFRowLayout({@required this.item, this.allTextChangeListener, this.allButtonClickListener, this.allItemSelectListener, Key key}): super(key: key);
  final NUIFRowItem item;
  final AllTextChangeListener allTextChangeListener;
  final AllButtonClickListener allButtonClickListener;
  final AllItemSelectListener allItemSelectListener;
  @override
  _NUIFRowLayoutState createState() => _NUIFRowLayoutState();
}

class _NUIFRowLayoutState extends NUIFState<NUIFRowLayout, NUIFRowItem> {
  @override
  void initState(){
    this.item = widget.item;
    super.initState();
  }

  Widget _renderComponent(NUIFormComponent comp){
    final flex = comp.getFlex();
    if(flex != null) {
      return Expanded(
        child: comp.toWidget(allTextChangeListener: widget.allTextChangeListener, allButtonClickListener: widget.allButtonClickListener, allItemSelectListener: widget.allItemSelectListener, key: widget.key),
        flex: (comp.getFlex()*10).toInt(),
      );
    }
    else{
      return Container(
        child: comp.toWidget(allTextChangeListener: widget.allTextChangeListener, allButtonClickListener: widget.allButtonClickListener, allItemSelectListener: widget.allItemSelectListener, key: widget.key),
        width: comp.width,
        height: comp.height,
      );
    }
  }

  List<Widget> _renderAllComponents(List<NUIFormComponent> items, double divider){
    List<Widget> views = List();
    bool addDivider = divider != null && divider > 0;
    NUICodeUtil.loop<NUIFormComponent>(items, (data, position, isFirst, isLast) {
      if(!isFirst && addDivider && data.show != false){
        views.add(VerticalDivider(width: divider, color: NUIColors.NUITransparent,));
      }

      views.add(Visibility(visible: data.show != false, child: _renderComponent(data)));
    });
    return views;
  }

  @override
  Widget build(BuildContext context) {
    logNUI("NUIFRow", "Rendering items for component ${item.inpId}");
    return Container(
      padding: item.padding,
      width: item.width,
      height: item.height,
      child: NUIRow(
        key: widget.key,
        crossAxisAlignment: item.crossAxisAlignment ?? CrossAxisAlignment.start,
        children: _renderAllComponents(item.items, item.divider),
      ),
    );
  }
}
